<?php

/**
 * @file
 * Functions for the 'user' link type for use with the Autolink module.
 */

/**
 * Callback function for building the user name select link form.
 */
function autolink_ui_user_select_method($edit) {
  $form = array();
  if (isset($edit['link_path'])) {
    $user = autolink_ui_user_load($edit['link_path']);
  }

  $users = autolink_ui_user_get_users();

  $select['select'] = t('--Select a user--');
  $select += $users;

  $form['argument'] = array(
    '#type' => 'select',
    '#title' => t('Users'),
    '#required' => TRUE,
    '#options' => $select,
    '#default_value' => isset($uid) ? $uid : 'select',
    '#description' => t('Links will link to the user profile of the selected user.'),
  );

  return $form;
}

/**
 * Callback function for building the user email link form.
 */
function autolink_ui_user_mail_method($edit) {
  $form = array();
  if (isset($edit['link_path'])) {
    $user = autolink_ui_user_load($edit['link_path']);
  }

  $form['argument'] = array(
    '#type' => 'textfield',
    '#title' => t('Enter a user email'),
    '#required' => TRUE,
    '#default_value' => isset($user) && !is_null($user) ? $user->mail : '',
  );

  return $form;
}

/**
 * Callback function for building the user name link form.
 */
function autolink_ui_user_name_method($edit) {
  $form = array();
  if (isset($edit['link_path'])) {
    $user = autolink_ui_user_load($edit['link_path']);
  }

  $form['argument'] = array(
    '#type' => 'textfield',
    '#title' => t('Enter a username'),
    '#required' => TRUE,
    '#default_value' => isset($user) && !is_null($user) ? $user->name : '',
  );

  return $form;
}

/**
 * Validation callback for the user name select form.
 */
function autolink_ui_user_select_method_validate($form_values) {
  if ($form_values['argument'] == 'select') {
    form_set_error('argument', t('You must select a user.'));
  }
}

/**
 * Validation callback for the user email form.
 */
function autolink_ui_user_mail_method_validate($form_values) {
  if (user_load(array('mail' => $form_values['argument'])) == FALSE) {
    form_set_error('argument', t('Cannot locate user data for user email \'%email\'.', array('%email' => $form_values['argument'])));
  }
}

/**
 * Validation callback for the user name form.
 */
function autolink_ui_user_name_method_validate($form_values) {
  if (user_load(array('name' => $form_values['argument'])) == FALSE) {
    form_set_error('argument', t('The user \'%user\' does not exist.', array('%user' => $form_values['argument'])));
  }
}

/**
 * Implements hook_autolink_LINK_TYPE_alter().
 *
 * Converts the link path to be saved to a search results page. Form data is used to
 * create a proper link path linking to the results page.
 *
 * @param $link
 *   An associative array of link form data.
 * @param $method
 *   The selection method used to create the link.
 *
 * @see autolink_ui_link_save()
 */
function autolink_ui_autolink_user_alter(&$link, $method) {
  if ($link['method'] == 'user_select') {
    $link['link_path'] = 'user/'. $link['argument'];
  }
  elseif ($link['method'] == 'user_name' || $link['method'] == 'user_mail') {
    $user = user_load(array($link['method'] => $link['argument']));
    $link['link_path'] = 'user/'. $user->uid;
  }
}

/**
 * Saves a new link definition based on user data.
 *
 * @see autolink_ui_user_operations()
 * @see autolink_ui_user_operations_name()
 */
function autolink_ui_user_save_link($user, $group) {
  $link->type = 'user';
  $link->group_type = $group;
  $link->keyword = $user->name;
  $link->link_path = 'user/'. $user->uid;
  $link->method = 'user_name';
  $link->case_sensitive = variable_get('autolink_case_sensitivity', 0);

  drupal_write_record('autolink_link', $link);
  module_invoke_all('autolink_link_insert', $link);
}

/**
 * Returns a list of users.
 *
 * @see autolink_ui_autolink_user_form_elements()
 */
function autolink_ui_user_get_users() {
  $users = array();
  $result = db_query('SELECT u.uid, u.name FROM {users} u WHERE u.uid <> 0');
  while ($user = db_fetch_object($result)) {
    $users[$user->uid] = $user->name;
  }
  asort($users);
  return $users;
}

/**
 * Loads a user from a parsed link path.
 */
function autolink_ui_user_load($path) {
  $args = explode('/', $path);
  $uid = $args[1];
  $user = is_numeric($uid) ? user_load(array('uid' => $uid)) : NULL;
  return $user;
}
